@@ -5,9 +5,8 @@ Permission to use, copy, modify, and distribute this software without fee
 is granted under the SciPy License.
 */
 #include <Python.h>
-
 #define PY_ARRAY_UNIQUE_SYMBOL _scipy_signal_ARRAY_API
-#include <numpy/noprefix.h>
+#include "numpy/ndarrayobject.h"
 
 #include "sigtools.h"
 #include <setjmp.h>
@@ -15,15 +14,6 @@ is granted under the SciPy License.
 
 #define PYERR(message) {PyErr_SetString(PyExc_ValueError, message); goto fail;}
 
-#define DATA(arr) ((arr)->data)
-#define DIMS(arr) ((arr)->dimensions)
-#define STRIDES(arr) ((arr)->strides)
-#define ELSIZE(arr) ((arr)->descr->elsize)
-#define OBJECTTYPE(arr) ((arr)->descr->type_num)
-#define BASEOBJ(arr) ((PyArrayObject *)((arr)->base))
-#define RANK(arr) ((arr)->nd)
-#define ISCONTIGUOUS(m) ((m)->flags & CONTIGUOUS)
-
 
 jmp_buf MALLOC_FAIL;
 
@@ -48,7 +38,7 @@ order filtering, however uses python-specific constructs in its guts
 and is therefore Python dependent.  This could be changed in a 
 straightforward way but I haven't done it for lack of time.*/
 
-static int index_out_of_bounds(intp *indices, intp *max_indices, int ndims) {
+static int index_out_of_bounds(npy_intp *indices, npy_intp *max_indices, int ndims) {
   int bad_index = 0, k = 0;
 
   while (!bad_index && (k++ < ndims)) {
@@ -65,9 +55,11 @@ static int index_out_of_bounds(intp *indices, intp *max_indices, int ndims) {
  * but probably with dim1 being the size of the "original, unsliced" array
  */
 
-static intp compute_offsets (uintp *offsets, intp *offsets2, intp *dim1, intp *dim2, intp *dim3, intp *mode_dep, int nd) {
+static npy_intp compute_offsets (npy_uintp *offsets, npy_intp *offsets2, npy_intp *dim1,
+                             npy_intp *dim2, npy_intp *dim3, npy_intp *mode_dep,
+                             int nd) {
   int k,i;
-  intp init_offset = 0;
+  npy_intp init_offset = 0;
 
   for (k = 0; k < nd - 1; k++) 
     {
@@ -100,7 +92,7 @@ static intp compute_offsets (uintp *offsets, intp *offsets2, intp *dim1, intp *d
 
 /* increment by 1 the index into an N-D array, doing the necessary
    carrying when the index reaches the dimension along that axis */ 
-static int increment(intp *ret_ind, int nd, intp *max_ind) {    
+static int increment(npy_intp *ret_ind, int nd, npy_intp *max_ind) {    
     int k, incr = 1;
     
     k = nd - 1;
@@ -223,7 +215,7 @@ static double freq_eval(int k, int n, double *grid, double *x, double *y, double
  */
 static int remez(double *dev, double des[], double grid[], double edge[],  
 	   double wt[], int ngrid, int nbands, int iext[], double alpha[],
-	   int nfcns, int itrmax, double *work, int dimsize)
+	   int nfcns, int itrmax, double *work, int dimsize, int *niter_out)
 		/* dev, iext, alpha                         are output types */
 		/* des, grid, edge, wt, ngrid, nbands, nfcns are input types */
 {
@@ -288,6 +280,7 @@ static int remez(double *dev, double des[], double grid[], double edge[],
 	}
 	if ( (*dev) <= devl ) {
 	    /* finished */
+	    *niter_out = niter;
 	    return -1;
 	}
 	devl = (*dev);
@@ -568,7 +561,9 @@ static double wate(double freq, double *fx, double *wtx, int lband, int jtype)
  *  Adpated from main() by Travis Oliphant
  */
 
-static int pre_remez(double *h2, int numtaps, int numbands, double *bands, double *response, double *weight, int type, int maxiter, int grid_density) {
+static int pre_remez(double *h2, int numtaps, int numbands, double *bands,
+                     double *response, double *weight, int type, int maxiter,
+                     int grid_density, int *niter_out) {
   
   int jtype, nbands, nfilt, lgrid, nz;
   int neg, nodd, nm1;
@@ -643,7 +638,11 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
 	    temp = grid[j];
 	    des[j] = eff(temp,fx,lband,jtype);
 	    wt[j] = wate(temp,fx,wtx,lband,jtype);
-	    if (++j > wrksize) { free(tempstor); return -1;} /* too many points, or too dense grid */
+	    if (++j > wrksize) {
+                /* too many points, or too dense grid */
+                free(tempstor);
+                return -1;
+            } 
 	    grid[j] = temp + delf;
 	} while (grid[j] <= fup);
 
@@ -698,7 +697,11 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
     nm1 = nfcns - 1;
     nz  = nfcns + 1;
 
-    if (remez(&dev, des, grid, edge, wt, ngrid, numbands, iext, alpha, nfcns, maxiter, work, dimsize) < 0) { free(tempstor); return -1; }
+    if (remez(&dev, des, grid, edge, wt, ngrid, numbands, iext, alpha, nfcns,
+              maxiter, work, dimsize, niter_out) < 0) {
+        free(tempstor);
+        return -1;
+    }
 
     /*
      * CALCULATE THE IMPULSE RESPONSE.
@@ -762,15 +765,17 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
 /* N-D Order Filtering. */
 
 
-static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2, char *sort_buffer, int nels2, int check, intp *loop_ind, intp *temp_ind, uintp *offset){ 
+static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2,
+                        char *sort_buffer, int nels2, int check,
+                        npy_intp *loop_ind, npy_intp *temp_ind, npy_uintp *offset){ 
   int i, k, incr = 1;
-  int ndims = ap1->nd;
-  intp *dims2 = ap2->dimensions;
-  intp *dims1 = ap1->dimensions;
-  intp is1 = ap1->descr->elsize;
-  intp is2 = ap2->descr->elsize;
-  char *ip2 = ap2->data;
-  int elsize = ap1->descr->elsize;
+  int ndims = PyArray_NDIM(ap1);
+  npy_intp *dims2 = PyArray_DIMS(ap2);
+  npy_intp *dims1 = PyArray_DIMS(ap1);
+  npy_intp is1 = PyArray_ITEMSIZE(ap1);
+  npy_intp is2 = PyArray_ITEMSIZE(ap2);
+  char *ip2 = PyArray_DATA(ap2);
+  int elsize = PyArray_ITEMSIZE(ap1);
   char *ptr;
 
   i = nels2;
@@ -787,11 +792,12 @@ static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2, char
     temp_ind[k]++;
 
     if (!(check && index_out_of_bounds(temp_ind,dims1,ndims)) && \
-	memcmp(ip2, ptr, ap2->descr->elsize)) { 
+	memcmp(ip2, ptr, PyArray_ITEMSIZE(ap2))) {
       memcpy(sort_buffer, ip1, elsize);
       sort_buffer += elsize;
     } 
-    incr = increment(loop_ind, ndims, dims2);  /* Returns number of N-D indices incremented. */
+    /* Returns number of N-D indices incremented. */
+    incr = increment(loop_ind, ndims, dims2);  
     ip2 += is2;
 
   }
@@ -804,22 +810,27 @@ int fname(type *ip1, type *ip2) { return *ip1 < *ip2 ? -1 : *ip1 == *ip2 ? 0 : 1
 
 COMPARE(DOUBLE_compare, double)
 COMPARE(FLOAT_compare, float)
-COMPARE(LONGDOUBLE_compare, longdouble)
-COMPARE(BYTE_compare, byte)
+COMPARE(LONGDOUBLE_compare, npy_longdouble)
+COMPARE(BYTE_compare, npy_byte)
 COMPARE(SHORT_compare, short)
 COMPARE(INT_compare, int)
 COMPARE(LONG_compare, long)
-COMPARE(LONGLONG_compare, longlong)
-COMPARE(UBYTE_compare, ubyte)
-COMPARE(USHORT_compare, ushort)
-COMPARE(UINT_compare, uint)
-COMPARE(ULONG_compare, ulong)
-COMPARE(ULONGLONG_compare, ulonglong)
+COMPARE(LONGLONG_compare, npy_longlong)
+COMPARE(UBYTE_compare, npy_ubyte)
+COMPARE(USHORT_compare, npy_ushort)
+COMPARE(UINT_compare, npy_uint)
+COMPARE(ULONG_compare, npy_ulong)
+COMPARE(ULONGLONG_compare, npy_ulonglong)
 
 
 int OBJECT_compare(PyObject **ip1, PyObject **ip2) {
-        /*return PyObject_Compare(*ip1, *ip2); */
-        return PyObject_RichCompareBool(*ip1, *ip2, Py_EQ) != 1;
+        /* PyObject_RichCompareBool returns -1 on error; not handled here */
+        if(PyObject_RichCompareBool(*ip1, *ip2, Py_LT) == 1)
+          return -1;
+        else if(PyObject_RichCompareBool(*ip1, *ip2, Py_EQ) == 1)
+          return 0;
+        else
+          return 1;
 }
 
 typedef int (*CompareFunction)(const void *, const void *);
@@ -836,16 +847,18 @@ CompareFunction compare_functions[] = \
 
 PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	PyArrayObject *ap1=NULL, *ap2=NULL, *ret=NULL;
-	intp *a_ind, *b_ind, *temp_ind, *mode_dep, *check_ind;
-	uintp *offsets, offset1;
-	intp *offsets2;
+	npy_intp *a_ind=NULL, *b_ind=NULL, *temp_ind=NULL, *mode_dep=NULL, *check_ind=NULL;
+	npy_uintp *offsets=NULL;
+	npy_intp *offsets2=NULL;
+	npy_uintp offset1;
 	int i, n2, n2_nonzero, k, check, incr = 1;
 	int typenum, bytes_in_array;
 	int is1, os;
-	char *op, *ap1_ptr, *ap2_ptr, *sort_buffer;
-	intp *ret_ind;
+	char *op, *ap1_ptr, *ap2_ptr, *sort_buffer=NULL;
+	npy_intp *ret_ind=NULL;
 	CompareFunction compare_func;
 	char *zptr=NULL;
+	PyArray_CopySwapFunc *copyswap;
 
 	/* Get Array objects from input */
 	typenum = PyArray_ObjectType(op1, 0);  
@@ -856,63 +869,75 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	ap2 = (PyArrayObject *)PyArray_ContiguousFromObject(op2, typenum, 0, 0);
 	if (ap2 == NULL) goto fail;
 
-	if (ap1->nd != ap2->nd) {
-	  PyErr_SetString(PyExc_ValueError, "All input arrays must have the same number of dimensions.");
+	if (PyArray_NDIM(ap1) != PyArray_NDIM(ap2)) {
+	    PyErr_SetString(PyExc_ValueError,
+                "All input arrays must have the same number of dimensions.");
 	  goto fail;
 	}
 
 	n2 = PyArray_Size((PyObject *)ap2);
 	n2_nonzero = 0;
-	ap2_ptr = ap2->data;
-	/* Find out the number of non-zero entries in domain (allows for
-	 *  different shapped rank-filters to be used besides just rectangles)
-	 */
+	ap2_ptr = PyArray_DATA(ap2);
+        /*
+         * Find out the number of non-zero entries in domain (allows for
+         * different shapped rank-filters to be used besides just rectangles)
+         */
 	zptr = PyArray_Zero(ap2);
 	if (zptr == NULL) goto fail;
 	for (k=0; k < n2; k++) {
-	  n2_nonzero += (memcmp(ap2_ptr,zptr,ap2->descr->elsize) != 0);
-	  ap2_ptr += ap2->descr->elsize;
+	  n2_nonzero += (memcmp(ap2_ptr,zptr,PyArray_ITEMSIZE(ap2)) != 0);
+	  ap2_ptr += PyArray_ITEMSIZE(ap2);
 	}
 
 	if ((order >= n2_nonzero) || (order < 0)) {
-	  PyErr_SetString(PyExc_ValueError, "Order must be non-negative and less than number of nonzero elements in domain.");
+	    PyErr_SetString(PyExc_ValueError,
+                "Order must be non-negative and less than number of nonzero elements in domain.");
 	  goto fail;
 	}
 	
-	ret = (PyArrayObject *)PyArray_SimpleNew(ap1->nd, ap1->dimensions, typenum);
+	ret = (PyArrayObject *)PyArray_SimpleNew(PyArray_NDIM(ap1),
+                                                 PyArray_DIMS(ap1),
+                                                 typenum);
 	if (ret == NULL) goto fail;
 	
-	compare_func = compare_functions[ap1->descr->type_num];
+	compare_func = compare_functions[PyArray_TYPE(ap1)];
 	if (compare_func == NULL) {
-		PyErr_SetString(PyExc_ValueError, 
-			"order_filterND not available for this type");
+	    PyErr_SetString(PyExc_ValueError, 
+                            "order_filterND not available for this type");
 		goto fail;
 	}
 
-	is1 = ap1->descr->elsize;
+	is1 = PyArray_ITEMSIZE(ap1);
 	
 	if (!(sort_buffer = malloc(n2_nonzero*is1))) goto fail;
 
-	op = ret->data; os = ret->descr->elsize;
+	os = PyArray_ITEMSIZE(ret);
+	op = PyArray_DATA(ret);
 
-	op = ret->data;
+	copyswap = PyArray_DESCR(ret)->f->copyswap;
 
-	bytes_in_array = ap1->nd*sizeof(intp);
+	bytes_in_array = PyArray_NDIM(ap1)*sizeof(npy_intp);
 	mode_dep = malloc(bytes_in_array);
-	for (k = 0; k < ap1->nd; k++) { 
-	  mode_dep[k] = -((ap2->dimensions[k]-1) >> 1);
+	if (mode_dep == NULL) goto fail;
+	for (k = 0; k < PyArray_NDIM(ap1); k++) { 
+	  mode_dep[k] = -((PyArray_DIMS(ap2)[k]-1) >> 1);
 	}	
 
-	b_ind = (intp *)malloc(bytes_in_array);  /* loop variables */
+	b_ind = (npy_intp *)malloc(bytes_in_array);  /* loop variables */
+	if (b_ind == NULL) goto fail;
 	memset(b_ind,0,bytes_in_array);
-	a_ind = (intp *)malloc(bytes_in_array);
-	ret_ind = (intp *)malloc(bytes_in_array);
+	a_ind = (npy_intp *)malloc(bytes_in_array);
+	ret_ind = (npy_intp *)malloc(bytes_in_array);
+	if (a_ind == NULL || ret_ind == NULL) goto fail;
 	memset(ret_ind,0,bytes_in_array);
-	temp_ind = (intp *)malloc(bytes_in_array);
-	check_ind = (intp*)malloc(bytes_in_array);
-	offsets = (uintp *)malloc(ap1->nd*sizeof(uintp));
-	offsets2 = (intp *)malloc(ap1->nd*sizeof(intp));
-	offset1 = compute_offsets(offsets,offsets2,ap1->dimensions,ap2->dimensions,ret->dimensions,mode_dep,ap1->nd);
+	temp_ind = (npy_intp *)malloc(bytes_in_array);
+	check_ind = (npy_intp*)malloc(bytes_in_array);
+	offsets = (npy_uintp *)malloc(PyArray_NDIM(ap1)*sizeof(npy_uintp));
+	offsets2 = (npy_intp *)malloc(PyArray_NDIM(ap1)*sizeof(npy_intp));
+	if (temp_ind == NULL || check_ind == NULL || offsets == NULL || offsets2 == NULL) goto fail;
+	offset1 = compute_offsets(offsets, offsets2, PyArray_DIMS(ap1),
+                                  PyArray_DIMS(ap2), PyArray_DIMS(ret),
+                                  mode_dep, PyArray_NDIM(ap1));
 	/* The filtering proceeds by looping through the output array
 	   and for each value filling a buffer from the 
 	   element-by-element product of the two input arrays.  The buffer
@@ -934,22 +959,27 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	PyDataMem_FREE(zptr);
 	zptr = PyArray_Zero(ap1);
 	if (zptr == NULL) goto fail;
-	ap1_ptr = ap1->data + offset1*is1;
-	for (k=0; k < ap1->nd; k++) {a_ind[k] = mode_dep[k]; check_ind[k] = ap1->dimensions[k] - ap2->dimensions[k] - mode_dep[k] - 1;}
-	a_ind[ap1->nd-1]--;
+	ap1_ptr = (char *)PyArray_DATA(ap1) + offset1*is1;
+	for (k=0; k < PyArray_NDIM(ap1); k++) {
+            a_ind[k] = mode_dep[k];
+            check_ind[k] = PyArray_DIMS(ap1)[k] - PyArray_DIMS(ap2)[k] - mode_dep[k] - 1;
+        }
+	a_ind[PyArray_NDIM(ap1)-1]--;
 	i = PyArray_Size((PyObject *)ret);
 	while (i--) {
-	  /* Zero out the sort_buffer (has effect of zero-padding
-	     on boundaries). Treat object arrays right.*/
+          /*
+           * Zero out the sort_buffer (has effect of zero-padding
+           * on boundaries). Treat object arrays right.
+           */
 	  ap2_ptr = sort_buffer;
 	  for (k=0; k < n2_nonzero; k++) {
   	    memcpy(ap2_ptr,zptr,is1);
 	    ap2_ptr += is1;
 	  }
 	    
-	  k = ap1->nd - 1;
+	  k = PyArray_NDIM(ap1) - 1;
 	  while(--incr) {
-	    a_ind[k] -= ret->dimensions[k] - 1;   /* Return to start */
+	    a_ind[k] -= PyArray_DIMS(ret)[k] - 1;   /* Return to start */
 	    k--;
 	  }
 	  ap1_ptr += offsets2[k]*is1;
@@ -957,15 +987,25 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	  memcpy(temp_ind, a_ind, bytes_in_array);
 
 	  check = 0; k = -1;
-	  while(!check && (++k < ap1->nd))
-	    check = check || (ret_ind[k] < -mode_dep[k]) || (ret_ind[k] > check_ind[k]);
+	  while(!check && (++k < PyArray_NDIM(ap1)))
+	    check = (check || (ret_ind[k] < -mode_dep[k]) || 
+                     (ret_ind[k] > check_ind[k]));
 
 	  fill_buffer(ap1_ptr,ap1,ap2,sort_buffer,n2,check,b_ind,temp_ind,offsets);
 	  qsort(sort_buffer, n2_nonzero, is1, compare_func);
-	  memcpy(op, sort_buffer + order*is1, os);
 
-	  incr = increment(ret_ind,ret->nd,ret->dimensions); /* increment index counter */
-	  op += os;   /* increment to next output index */
+	  /*
+	   * Use copyswap for correct refcounting with object arrays
+	   * (sort_buffer has borrowed references, op owns references). Note
+	   * also that os == PyArray_ITEMSIZE(ret) and we are copying a single
+	   * scalar here.
+	   */
+	  copyswap(op, sort_buffer + order*is1, 0, NULL);
+
+          /* increment index counter */
+	  incr = increment(ret_ind,PyArray_NDIM(ret),PyArray_DIMS(ret)); 
+          /* increment to next output index */
+	  op += os;
 
 	}
 	free(b_ind); free(a_ind); free(ret_ind);
@@ -981,6 +1021,15 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 
 fail:
 	if (zptr) PyDataMem_FREE(zptr);
+	free(sort_buffer);
+	free(mode_dep);
+	free(b_ind);
+	free(a_ind);
+	free(ret_ind);
+	free(temp_ind);
+	free(check_ind);
+    free(offsets);
+	free(offsets2);
 	Py_XDECREF(ap1);
 	Py_XDECREF(ap2);
 	Py_XDECREF(ret);
@@ -996,19 +1045,20 @@ static char doc_correlateND[] = "out = _correlateND(a,kernel,mode) \n\n   mode =
 
 static char doc_convolve2d[] = "out = _convolve2d(in1, in2, flip, mode, boundary, fillvalue)";
 
-extern int pylab_convolve_2d(char*,intp*,char*,intp*,char*,intp*,intp*,intp*,int,char*);
+extern int pylab_convolve_2d(char*, npy_intp*, char*, npy_intp*, char*,
+                             npy_intp*, npy_intp*, npy_intp*, int, char*);
 
 static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 
     PyObject *in1=NULL, *in2=NULL, *fill_value=NULL;
     int mode=2, boundary=0, typenum, flag, flip=1, ret;
-    intp *aout_dimens=NULL, *dims=NULL;
-    char zeros[32];  /* Zeros */
+    npy_intp *aout_dimens=NULL;
     int i;
     PyArrayObject *ain1=NULL, *ain2=NULL, *aout=NULL;
-    PyArrayObject *afill=NULL, *newfill=NULL;
+    PyArrayObject *afill=NULL;
 
-    if (!PyArg_ParseTuple(args, "OO|iiiO", &in1, &in2, &flip, &mode, &boundary, &fill_value)) {
+    if (!PyArg_ParseTuple(args, "OO|iiiO", &in1, &in2, &flip, &mode, &boundary,
+                          &fill_value)) {
         return NULL;
     }
 
@@ -1021,38 +1071,70 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 
     if ((boundary != PAD) && (boundary != REFLECT) && (boundary != CIRCULAR))
       PYERR("Incorrect boundary value.");
-    if (boundary == PAD) {
-	if (fill_value == NULL) {
-	    newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum, zeros);
-	}
-	else {
-	    afill = (PyArrayObject *)PyArray_FromObject(fill_value, NPY_CDOUBLE, 0, 0);
-	    if (afill == NULL) goto fail;
-	    newfill = (PyArrayObject *)PyArray_Cast(afill, typenum);
-	}
-	if (newfill == NULL) goto fail;
+
+    if ((boundary == PAD) & (fill_value != NULL)) {
+        afill = (PyArrayObject *)PyArray_FromObject(fill_value, typenum, 0, 0);
+        if (afill == NULL) {
+            /* For backwards compatibility try go via complex */
+            PyArrayObject *tmp;
+            PyErr_Clear();
+            tmp = (PyArrayObject *)PyArray_FromObject(fill_value,
+                                                      NPY_CDOUBLE, 0, 0);
+            if (tmp == NULL) goto fail;
+            afill = (PyArrayObject *)PyArray_Cast(tmp, typenum);
+            Py_DECREF(tmp);
+            if (afill == NULL) goto fail;
+            /* Deprecated 2017-07, scipy version 1.0.0 */
+            if (DEPRECATE("could not cast `fillvalue` directly to the output "
+                          "type (it was first converted to complex). "
+                          "This is deprecated and will raise an error in the "
+                          "future.") < 0) {
+                goto fail;
+            }
+        }
+        if (PyArray_SIZE(afill) != 1) {
+            if (PyArray_SIZE(afill) == 0) {
+                PyErr_SetString(PyExc_ValueError,
+                                "`fillvalue` cannot be an empty array.");
+                goto fail;
+            }
+            /* Deprecated 2017-07, scipy version 1.0.0 */
+            if (DEPRECATE("`fillvalue` must be scalar or an array with "
+                          "one element. "
+                          "This will raise an error in the future.") < 0) {
+                goto fail;
+            }
+        }
     }
     else {
-	newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum, zeros);
-	if (newfill == NULL) goto fail;
+        /* Create a zero filled array */
+        afill = (PyArrayObject *)PyArray_ZEROS(0, NULL, typenum, 0);
+        if (afill == NULL) goto fail;
     }
 
-    aout_dimens = malloc(ain1->nd*sizeof(intp));
+    aout_dimens = malloc(PyArray_NDIM(ain1)*sizeof(npy_intp));
+    if (aout_dimens == NULL) goto fail;
     switch(mode & OUTSIZE_MASK) {
     case VALID:
-	for (i = 0; i < ain1->nd; i++) { 
-	    aout_dimens[i] = ain1->dimensions[i] - ain2->dimensions[i] + 1;
+	for (i = 0; i < PyArray_NDIM(ain1); i++) { 
+	    aout_dimens[i] = PyArray_DIMS(ain1)[i] - PyArray_DIMS(ain2)[i] + 1;
 	    if (aout_dimens[i] < 0) {
-		PyErr_SetString(PyExc_ValueError, "no part of the output is valid, use option 1 (same) or 2 (full) for third argument");
+		PyErr_SetString(PyExc_ValueError,
+                    "no part of the output is valid, use option 1 (same) or 2 "
+                    "(full) for third argument");
 		goto fail;
 	    }
 	}
 	break;
     case SAME:
-	for (i = 0; i < ain1->nd; i++) { aout_dimens[i] = ain1->dimensions[i];}
+	for (i = 0; i < PyArray_NDIM(ain1); i++) {
+            aout_dimens[i] = PyArray_DIMS(ain1)[i];
+        }
 	break;
     case FULL:
-	for (i = 0; i < ain1->nd; i++) { aout_dimens[i] = ain1->dimensions[i] + ain2->dimensions[i] - 1;}
+	for (i = 0; i < PyArray_NDIM(ain1); i++) {
+            aout_dimens[i] = PyArray_DIMS(ain1)[i] + PyArray_DIMS(ain2)[i] - 1;
+        }
 	break;
     default: 
 	PyErr_SetString(PyExc_ValueError, 
@@ -1060,22 +1142,23 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 	goto fail;
     }
 	
-    aout = (PyArrayObject *)PyArray_SimpleNew(ain1->nd, aout_dimens, typenum);
+    aout = (PyArrayObject *)PyArray_SimpleNew(PyArray_NDIM(ain1), aout_dimens,
+                                              typenum);
     if (aout == NULL) goto fail;
 
     flag = mode + boundary + (typenum << TYPE_SHIFT) + \
       (flip != 0) * FLIP_MASK;
     
-    ret = pylab_convolve_2d (DATA(ain1),      /* Input data Ns[0] x Ns[1] */
-		             STRIDES(ain1),   /* Input strides */
-		             DATA(aout),      /* Output data */
-		             STRIDES(aout),   /* Ouput strides */
-		             DATA(ain2),      /* coefficients in filter */
-		             STRIDES(ain2),   /* coefficients strides */ 
-		             DIMS(ain2),      /* Size of kernel Nwin[2] */
-			     DIMS(ain1),      /* Size of image Ns[0] x Ns[1] */
-		             flag,            /* convolution parameters */
-		             DATA(newfill));  /* fill value */
+    ret = pylab_convolve_2d (PyArray_DATA(ain1),      /* Input data Ns[0] x Ns[1] */
+		             PyArray_STRIDES(ain1),   /* Input strides */
+		             PyArray_DATA(aout),      /* Output data */
+		             PyArray_STRIDES(aout),   /* Output strides */
+		             PyArray_DATA(ain2),      /* coefficients in filter */
+		             PyArray_STRIDES(ain2),   /* coefficients strides */ 
+		             PyArray_DIMS(ain2),      /* Size of kernel Nwin[2] */
+			     PyArray_DIMS(ain1),      /* Size of image Ns[0] x Ns[1] */
+		             flag,                    /* convolution parameters */
+		             PyArray_DATA(afill));    /* fill value */
 
 
     switch (ret) {
@@ -1084,7 +1167,6 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
       Py_DECREF(ain1);
       Py_DECREF(ain2);
       Py_XDECREF(afill);
-      Py_XDECREF(newfill);
       return (PyObject *)aout;
       break;
     case -5:
@@ -1111,7 +1193,6 @@ fail:
     Py_XDECREF(ain2);
     Py_XDECREF(aout);
     Py_XDECREF(afill);
-    Py_XDECREF(newfill);
     return NULL;
 }
 
@@ -1119,7 +1200,8 @@ fail:
 
 static char doc_order_filterND[] = "out = _order_filterND(a,domain,order)";
 
-static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy), PyObject *args) {
+static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy), 
+                                         PyObject *args) {
 	PyObject *domain, *a0;
 	int order=0;
 	
@@ -1129,108 +1211,120 @@ static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy), PyObject *
 }
 
 
+static char doc_remez[] =
+    "h = _remez(numtaps, bands, des, weight, type, fs, maxiter, grid_density)\n"
+    "  returns the optimal (in the Chebyshev/minimax sense) FIR filter impulse\n"
+    "  response given a set of band edges, the desired response on those bands,\n"
+    "  and the weight given to the error in those bands.  Bands is a monotonic\n"
+    "  vector with band edges given in frequency domain where fs is the sampling\n"
+    "  frequency.";
 
-static char doc_remez[] = "h = _remez(numtaps, bands, des, weight, type, Hz, maxiter, grid_density) \n  returns the optimal (in the Chebyshev/minimax sense) FIR filter impulse \n  response given a set of band edges, the desired response on those bands,\n  and the weight given to the error in those bands.  Bands is a monotonic\n   vector with band edges given in frequency domain where Hz is the sampling\n   frequency.";
- 
-static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
-        PyObject *bands, *des, *weight;
-        int k, numtaps, numbands, type = BANDPASS, err; 
-	PyArrayObject *a_bands=NULL, *a_des=NULL, *a_weight=NULL;
-	PyArrayObject *h=NULL;
-	intp ret_dimens; int maxiter = 25, grid_density = 16;
-	double oldvalue, *dptr, Hz = 1.0;
-        char mystr[255];
-       
+static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args)
+{
+    PyObject *bands, *des, *weight;
+    int k, numtaps, numbands, type = BANDPASS, err; 
+    PyArrayObject *a_bands=NULL, *a_des=NULL, *a_weight=NULL;
+    PyArrayObject *h=NULL;
+    npy_intp ret_dimens; int maxiter = 25, grid_density = 16;
+    double oldvalue, *dptr, fs = 1.0;
+    char mystr[255];
+    int niter = -1;
 
+    if (!PyArg_ParseTuple(args, "iOOO|idii", &numtaps, &bands, &des, &weight, 
+                          &type, &fs, &maxiter, &grid_density)) {
+        return NULL;
+    }
 
-        if (!PyArg_ParseTuple(args, "iOOO|idii", &numtaps, &bands, &des, &weight, &type, &Hz, &maxiter, &grid_density))
-	    return NULL;
-
-	if (type != BANDPASS && type != DIFFERENTIATOR && type != HILBERT) {
-	  PyErr_SetString(PyExc_ValueError,
-			  "The type must be BANDPASS, DIFFERENTIATOR, or HILBERT.");
-	  return NULL;
-	}
-	
-	if (numtaps < 2) {
-	  PyErr_SetString(PyExc_ValueError,
-			  "The number of taps must be greater than 1.");
-	  return NULL;
-	}
-	
-	
-	a_bands = (PyArrayObject *)PyArray_ContiguousFromObject(bands, NPY_DOUBLE,1,1);
-	if (a_bands == NULL) goto fail;
-	a_des = (PyArrayObject *)PyArray_ContiguousFromObject(des, NPY_DOUBLE,1,1);
-	if (a_des == NULL) goto fail;
-	a_weight = (PyArrayObject *)PyArray_ContiguousFromObject(weight, NPY_DOUBLE,1,1);
-	if (a_weight == NULL) goto fail;
-
-
-	numbands = a_des->dimensions[0];
-	if ((a_bands->dimensions[0] != 2*numbands) || (a_weight->dimensions[0] != numbands)) {
-	  PyErr_SetString(PyExc_ValueError,
-			  "The inputs desired and weight must have same length.\n  The input bands must have twice this length.");
-	  goto fail;
+    if (type != BANDPASS && type != DIFFERENTIATOR && type != HILBERT) {
+        PyErr_SetString(PyExc_ValueError,
+                        "The type must be BANDPASS, DIFFERENTIATOR, or HILBERT.");
+        return NULL;
 	}
 
-      /* Check the bands input to see if it is monotonic, divide by 
-	 Hz to take from range 0 to 0.5 and check to see if in that range */ 
+    if (numtaps < 2) {
+        PyErr_SetString(PyExc_ValueError,
+                        "The number of taps must be greater than 1.");
+        return NULL;
+    }
 
-	dptr = (double *)a_bands->data;
-	oldvalue = 0;
-	for (k=0; k < 2*numbands; k++) {
-	  if (*dptr < oldvalue) {
+    a_bands = (PyArrayObject *)PyArray_ContiguousFromObject(bands, NPY_DOUBLE,1,1);
+    if (a_bands == NULL) goto fail;
+    a_des = (PyArrayObject *)PyArray_ContiguousFromObject(des, NPY_DOUBLE,1,1);
+    if (a_des == NULL) goto fail;
+    a_weight = (PyArrayObject *)PyArray_ContiguousFromObject(weight, NPY_DOUBLE,1,1);
+    if (a_weight == NULL) goto fail;
+
+    numbands = PyArray_DIMS(a_des)[0];
+    if ((PyArray_DIMS(a_bands)[0] != 2*numbands) || 
+        (PyArray_DIMS(a_weight)[0] != numbands)) {
 	    PyErr_SetString(PyExc_ValueError,
-			  "Bands must be monotonic starting at zero.");
-	    goto fail;
-	  }
-	  if (*dptr * 2 > Hz) {
-	    PyErr_SetString(PyExc_ValueError,
-			  "Band edges should be less than 1/2 the sampling frequency");
-	    goto fail;
-	  }
-	  oldvalue = *dptr;
-	  *dptr = oldvalue / Hz;  /* Change so that sampling frequency is 1.0 */
-	  dptr++;
-	}
+                        "The inputs desired and weight must have same length.\n  "
+                        "The input bands must have twice this length.");
+        goto fail;
+    }
 
-	ret_dimens = numtaps;
-	h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, NPY_DOUBLE);
-	if (h == NULL) goto fail;
-
-	err=pre_remez((double *)h->data, numtaps, numbands, (double *)a_bands->data, (double *)a_des->data, (double *)a_weight->data, type, maxiter, grid_density);
-        if (err < 0) {
-	  if (err == -1) {
-            sprintf(mystr,"Failure to converge after %d iterations.\n      Design may still be correct.",maxiter);
-	    PyErr_SetString(PyExc_ValueError, mystr);
-	    goto fail;
-	  }
-	  else if (err == -2) {
-	    PyErr_NoMemory();
+    /* 
+     * Check the bands input to see if it is monotonic, divide by 
+     * fs to take from range 0 to 0.5 and check to see if in that range
+     */ 
+    dptr = (double *)PyArray_DATA(a_bands);
+    oldvalue = 0;
+    for (k=0; k < 2*numbands; k++) {
+        if (*dptr < oldvalue) {
+            PyErr_SetString(PyExc_ValueError,
+                            "Bands must be monotonic starting at zero.");
             goto fail;
-	  }
-	}
+        }
+        if (*dptr * 2 > fs) {
+            PyErr_SetString(PyExc_ValueError,
+                            "Band edges should be less than 1/2 the sampling frequency");
+            goto fail;
+        }
+        oldvalue = *dptr;
+        *dptr = oldvalue / fs;  /* Change so that sampling frequency is 1.0 */
+        dptr++;
+    }
 
-	Py_DECREF(a_bands);
-	Py_DECREF(a_des);
-	Py_DECREF(a_weight);
+    ret_dimens = numtaps;
+    h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, NPY_DOUBLE);
+    if (h == NULL) goto fail;
+
+    err = pre_remez((double *)PyArray_DATA(h), numtaps, numbands, 
+                    (double *)PyArray_DATA(a_bands),
+                    (double *)PyArray_DATA(a_des),
+                    (double *)PyArray_DATA(a_weight),
+                    type, maxiter, grid_density, &niter);
+    if (err < 0) {
+        if (err == -1) {
+            sprintf(mystr, "Failure to converge at iteration %d, try reducing transition band width.\n", niter);
+	        PyErr_SetString(PyExc_ValueError, mystr);
+	        goto fail;
+        }
+        else if (err == -2) {
+            PyErr_NoMemory();
+            goto fail;
+        }
+    }
+
+    Py_DECREF(a_bands);
+    Py_DECREF(a_des);
+    Py_DECREF(a_weight);
 
 	return PyArray_Return(h);
 
- fail:
-	Py_XDECREF(a_bands);
-	Py_XDECREF(a_des);
-	Py_XDECREF(a_weight);
-	Py_XDECREF(h);
-	return NULL;
+fail:
+    Py_XDECREF(a_bands);
+    Py_XDECREF(a_des);
+    Py_XDECREF(a_weight);
+    Py_XDECREF(h);
+    return NULL;
 }
-   
+
 static char doc_median2d[] = "filt = _median2d(data, size)";
 
-extern void f_medfilt2(float*,float*,intp*,intp*);
-extern void d_medfilt2(double*,double*,intp*,intp*);
-extern void b_medfilt2(unsigned char*,unsigned char*,intp*,intp*);
+extern void f_medfilt2(float*,float*,npy_intp*,npy_intp*);
+extern void d_medfilt2(double*,double*,npy_intp*,npy_intp*);
+extern void b_medfilt2(unsigned char*,unsigned char*,npy_intp*,npy_intp*);
 
 static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
 {
@@ -1238,7 +1332,7 @@ static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
     int typenum;
     PyArrayObject *a_image=NULL, *a_size=NULL;
     PyArrayObject *a_out=NULL;
-    intp Nwin[2] = {3,3};
+    npy_intp Nwin[2] = {3,3};
 
     if (!PyArg_ParseTuple(args, "O|O", &image, &size)) return NULL;
 
@@ -1249,13 +1343,13 @@ static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
     if (size != NULL) {
 	a_size = (PyArrayObject *)PyArray_ContiguousFromObject(size, NPY_INTP, 1, 1);
 	if (a_size == NULL) goto fail;
-	if ((RANK(a_size) != 1) || (DIMS(a_size)[0] < 2)) 
+	if ((PyArray_NDIM(a_size) != 1) || (PyArray_DIMS(a_size)[0] < 2)) 
 	    PYERR("Size must be a length two sequence");
-	Nwin[0] = ((intp *)DATA(a_size))[0];
-	Nwin[1] = ((intp *)DATA(a_size))[1];
+	Nwin[0] = ((npy_intp *)PyArray_DATA(a_size))[0];
+	Nwin[1] = ((npy_intp *)PyArray_DATA(a_size))[1];
     }  
 
-    a_out = (PyArrayObject *)PyArray_SimpleNew(2,DIMS(a_image),typenum);
+    a_out = (PyArrayObject *)PyArray_SimpleNew(2, PyArray_DIMS(a_image), typenum);
     if (a_out == NULL) goto fail;
 
     if (setjmp(MALLOC_FAIL)) {
@@ -1264,16 +1358,22 @@ static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
     else {
 	switch (typenum) {
 	case NPY_UBYTE:
-	    b_medfilt2((unsigned char *)DATA(a_image), (unsigned char *)DATA(a_out), Nwin, DIMS(a_image));
+	    b_medfilt2((unsigned char *)PyArray_DATA(a_image),
+                       (unsigned char *)PyArray_DATA(a_out),
+                       Nwin, PyArray_DIMS(a_image));
 	    break;
 	case NPY_FLOAT:
-	    f_medfilt2((float *)DATA(a_image), (float *)DATA(a_out), Nwin, DIMS(a_image));
+	    f_medfilt2((float *)PyArray_DATA(a_image),
+                       (float *)PyArray_DATA(a_out), Nwin,
+                       PyArray_DIMS(a_image));
 	    break;
 	case NPY_DOUBLE:
-	    d_medfilt2((double *)DATA(a_image), (double *)DATA(a_out), Nwin, DIMS(a_image));
+	    d_medfilt2((double *)PyArray_DATA(a_image),
+                       (double *)PyArray_DATA(a_out), Nwin,
+                       PyArray_DIMS(a_image));
 	    break;
 	default:
-	  PYERR("2D median filter only supports Int8, Float32, and Float64.");
+	  PYERR("2D median filter only supports uint8, float32, and float64.");
 	}
     }
 
@@ -1305,7 +1405,6 @@ static struct PyMethodDef toolbox_module_methods[] = {
 	{NULL, NULL, 0, NULL}		/* sentinel */
 };
 
-#if PY_VERSION_HEX >= 0x03000000
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "sigtools",
@@ -1328,29 +1427,3 @@ PyObject *PyInit_sigtools(void)
 
     return m;
 }
-#else
-/* Initialization function for the module (*must* be called initsigtools) */
-
-PyMODINIT_FUNC initsigtools(void) {
-    /* Create the module and add the functions */
-    Py_InitModule("sigtools", toolbox_module_methods);
-
-	/* Import the C API function pointers for the Array Object*/
-	import_array();
-
-	/* Make sure the multiarraymodule is loaded so that the zero
-	   and one objects are defined */
-	/* XXX: This should be updated for scipy. I think it's pulling in 
-	   Numeric's multiarray. */
-	PyImport_ImportModule("numpy.core.multiarray");
-	/* { PyObject *multi = PyImport_ImportModule("multiarray"); } */
-
-    scipy_signal_sigtools_linear_filter_module_init();
-
-	/* Check for errors */
-	if (PyErr_Occurred()) {
-	  PyErr_Print();
-	  Py_FatalError("can't initialize module array");
-	}
-}
-#endif
